# -*- coding: utf-8 -*-

# Copyright 2008-2011 Richard Dymond (rjdymond@gmail.com)
#
# This file is part of SkoolKit.
#
# SkoolKit is free software: you can redistribute it and/or modify it under the
# terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# SkoolKit is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
# A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# SkoolKit. If not, see <http://www.gnu.org/licenses/>.

import cgi
from microsphere import MicrosphereGame, MicrosphereHtmlWriter, Udg
from skoollib import MAIN_CODE_ID
from skoolhtml import Table, Cell

class SkoolDaze(MicrosphereGame):
    def __init__(self, skool_parser, ref_parser, code_id=MAIN_CODE_ID):
        MicrosphereGame.__init__(self, skool_parser, ref_parser, code_id)
        self.min_lesson = 224
        self.max_lesson = 255
        self.min_character = 152
        self.max_character = 169
        self.font_address = 55552
        self.char_buf_descs = self._get_char_buf_descs()
        self.keypress_routines = self.get_data_dictionary('KeypressRoutines')
        self.as_descs = self.get_data_dictionary('AnimatoryStates')
        self.characters = self.get_data_dictionary('Characters')
        self.lesson_descs = self.get_data_dictionary('Lessons')
        self._calculate_tap_index(65152, 222)

    def get_skool_udg(self, y, x, show_chars=False):
        ref_page = y + 152
        attr_addr = x + 128 + 256 * ref_page
        ref_addr = x + 256 * ref_page
        ref = self.snapshot[ref_addr]
        attr = self.snapshot[attr_addr]
        udg_page = 128 + 8 * (x // 32)
        udg_addr = ref + 256 * udg_page
        udg_bytes = self.snapshot[udg_addr:udg_addr + 2048:256]
        skool_udg = Udg(attr, udg_bytes, attr_addr=attr_addr, ref_addr=ref_addr, ref=ref, udg_page=udg_page, x=x, y=y)
        if show_chars:
            self._superimpose_sprite_udgs(skool_udg, x, y)
        return skool_udg

    def get_sprite_udg(self, state, attr, row, col, udg_page=None):
        if udg_page is None:
            udg_page = 201 if 71 < (state & 127) < 104 else 185
        ref_page = 173 + col * 4 + row
        return self._get_sprite_udg(state, attr, ref_page, udg_page)

    def get_initial_states(self):
        initial_states = []
        for c in range(18) + [20]:
            initial_states.append((self.snapshot[56064 + c], self.snapshot[56320 + c], self.snapshot[56576 + c]))
        return initial_states

class SkoolDazeWriter(MicrosphereHtmlWriter):
    def keypress_table(self, cwd):
        keypress_routines = self.skool.keypress_routines
        snapshot = self.skool.snapshot

        table = Table('keytable', 'Code', 'Key', 'Address', 'Offset', 'Lookup', 'Keycode', 'Routine', 'Purpose')
        for c in range(32, 128):
            key = '&#169;' if c == 127 else cgi.escape(chr(c))
            address = c + 26624
            offset = snapshot[address]
            lookup = offset + 26624
            key_byte = snapshot[lookup]
            keycode = 'SPACE' if key_byte == 32 else chr(key_byte)
            routine = snapshot[lookup + 1] + 256 * snapshot[lookup + 2]
            purpose = keypress_routines[routine]
            routine_link = self.expand('#R%i' % routine, cwd)
            row = []
            row.append(Cell(c, "keyIndex"))
            row.append(Cell(key, "key"))
            row.append(Cell(address, "keyAddress"))
            row.append(Cell(offset, "data"))
            row.append(Cell(lookup, "keyAddress"))
            row.append(Cell(keycode, "centre"))
            row.append(Cell(routine_link, "keyAddress"))
            row.append(Cell(purpose, "purpose"))
            table.add_row(row)

        return table.html()

    def get_playarea_tile_text(self, tile):
        text = ['<i>ref_addr</i>=(x,y+152)=%i' % tile.ref_addr]
        text.append('<i>ref</i>=%i, <i>udg_page</i>=%i' % (tile.ref, tile.udg_page))
        text.append('<i>udg_addr</i>=(<i>ref</i>,<i>udg_page</i>)=%i' % (tile.ref + 256 * tile.udg_page))
        text.append('<i>attr_addr</i>=(x+128,y+152)=%i' % tile.attr_addr)
        text.append('<i>attr</i>=%i' % tile.attr)
        return text

    def _animatory_state_row(self, cwd, rownum):
        lines = ['<tr>']
        for state in (rownum, rownum + 128):
            lines.append('<td>')
            lines.append('<a name="%i" />%s' % (state, self.as_img(cwd, state, 2, 1, 120, 185)))
            lines.append('</td>')
        for state in (rownum, rownum + 128):
            lines.append('<td>')
            lines.append('<a name="%i" />%s' % (state, self.as_img(cwd, state, 2, 1, 120, 201)))
            lines.append('</td>')
        lines.append('<td>%i, %i: %s</td>' % (rownum, rownum + 128, self.skool.as_descs.get(rownum, '-')))
        lines.append('</tr>')
        return lines

    def _get_animatory_state_tiles_row(self, n):
        states = ((n, 185), (n + 128, 185), (n, 201), (n + 128, 201))
        states_desc = '%i, %i: %s' % (n, n + 128, self.skool.as_descs[n])
        return [(states, states_desc)]
