# -*- coding: utf-8 -*-
import os
from os.path import join
from shutil import rmtree
import unittest
import nose
from lxml import etree
from skoolkittest import SkoolKitTestCase

ODIR = 'html'
XHTML_XSD = 'xhtml1-strict.xsd'

OUTPUT = """Using 'null' image library
Creating directory html
Using ref file: ../src/sd.ref
Parsing ../src/sd.skool
Creating directory html/skool_daze
Copying ../src/style.css to html/skool_daze/style.css
Copying ../src/skool.js to html/skool_daze/skool.js
  Wrote skool_daze/images/logo.png
  Writing disassembly files in skool_daze/asm
  Writing skool_daze/maps/all.html
  Writing skool_daze/maps/routines.html
  Writing skool_daze/maps/data.html
  Writing skool_daze/maps/messages.html
  Writing skool_daze/graphics/playarea.html
  Writing skool_daze/graphics/patiles/patiles.html
  Writing skool_daze/graphics/asstart.html
  Writing skool_daze/graphics/as.html
  Writing skool_daze/graphics/astiles/astiles.html
  Writing skool_daze/buffers/cbuffer.html
  Writing skool_daze/lessons/timetables.html
  Writing skool_daze/lessons/index.html
  Writing skool_daze/lessons/224.html
  Writing skool_daze/lessons/225.html
  Writing skool_daze/lessons/226.html
  Writing skool_daze/lessons/227.html
  Writing skool_daze/lessons/228.html
  Writing skool_daze/lessons/229.html
  Writing skool_daze/lessons/230.html
  Writing skool_daze/lessons/231.html
  Writing skool_daze/lessons/232.html
  Writing skool_daze/lessons/233.html
  Writing skool_daze/lessons/234.html
  Writing skool_daze/lessons/235.html
  Writing skool_daze/lessons/236.html
  Writing skool_daze/lessons/237.html
  Writing skool_daze/lessons/238.html
  Writing skool_daze/lessons/239.html
  Writing skool_daze/lessons/240.html
  Writing skool_daze/lessons/241.html
  Writing skool_daze/lessons/242.html
  Writing skool_daze/lessons/243.html
  Writing skool_daze/lessons/244.html
  Writing skool_daze/lessons/245.html
  Writing skool_daze/lessons/246.html
  Writing skool_daze/lessons/247.html
  Writing skool_daze/lessons/248.html
  Writing skool_daze/lessons/249.html
  Writing skool_daze/lessons/250.html
  Writing skool_daze/lessons/251.html
  Writing skool_daze/lessons/252.html
  Writing skool_daze/lessons/253.html
  Writing skool_daze/lessons/254.html
  Writing skool_daze/lessons/255.html
  Writing skool_daze/tables/keys.html
  Writing skool_daze/buffers/gbuffer.html
  Writing skool_daze/graphics/graphics.html
  Writing skool_daze/graphics/glitches.html
  Writing skool_daze/reference/bugs.html
  Writing skool_daze/reference/facts.html
  Writing skool_daze/reference/glossary.html
  Writing skool_daze/reference/pokes.html
  Parsing ../src/sd-load.skool
    Writing skool_daze/load/load.html
    Writing disassembly files in skool_daze/load
  Parsing ../src/sd-save.skool
    Writing skool_daze/save/save.html
    Writing disassembly files in skool_daze/save
  Parsing ../src/sd-start.skool
    Writing skool_daze/start/start.html
    Writing disassembly files in skool_daze/start
  Writing skool_daze/index.html
Using ref file: ../src/bts.ref
Parsing ../src/bts.skool
Creating directory html/back_to_skool
Copying ../src/style.css to html/back_to_skool/style.css
Copying ../src/skool.js to html/back_to_skool/skool.js
  Wrote back_to_skool/images/logo.png
  Writing disassembly files in back_to_skool/asm
  Writing back_to_skool/maps/all.html
  Writing back_to_skool/maps/routines.html
  Writing back_to_skool/maps/data.html
  Writing back_to_skool/maps/messages.html
  Writing back_to_skool/graphics/playarea.html
  Writing back_to_skool/graphics/patiles/patiles.html
  Writing back_to_skool/graphics/asstart.html
  Writing back_to_skool/graphics/as.html
  Writing back_to_skool/graphics/astiles/astiles.html
  Writing back_to_skool/buffers/cbuffer.html
  Writing back_to_skool/lessons/timetables.html
  Writing back_to_skool/lessons/index.html
  Writing back_to_skool/lessons/37.html
  Writing back_to_skool/lessons/38.html
  Writing back_to_skool/lessons/39.html
  Writing back_to_skool/lessons/40.html
  Writing back_to_skool/lessons/41.html
  Writing back_to_skool/lessons/42.html
  Writing back_to_skool/lessons/43.html
  Writing back_to_skool/lessons/44.html
  Writing back_to_skool/lessons/45.html
  Writing back_to_skool/lessons/46.html
  Writing back_to_skool/lessons/47.html
  Writing back_to_skool/lessons/48.html
  Writing back_to_skool/lessons/49.html
  Writing back_to_skool/lessons/50.html
  Writing back_to_skool/lessons/51.html
  Writing back_to_skool/lessons/52.html
  Writing back_to_skool/lessons/53.html
  Writing back_to_skool/lessons/54.html
  Writing back_to_skool/lessons/55.html
  Writing back_to_skool/lessons/56.html
  Writing back_to_skool/lessons/57.html
  Writing back_to_skool/lessons/58.html
  Writing back_to_skool/lessons/59.html
  Writing back_to_skool/tables/keys.html
  Writing back_to_skool/buffers/gbuffer.html
  Writing back_to_skool/graphics/graphics.html
  Writing back_to_skool/graphics/glitches.html
  Writing back_to_skool/reference/bugs.html
  Writing back_to_skool/reference/facts.html
  Writing back_to_skool/reference/glossary.html
  Writing back_to_skool/reference/pokes.html
  Parsing ../src/bts-load.skool
    Writing back_to_skool/load/load.html
    Writing disassembly files in back_to_skool/load
  Parsing ../src/bts-save.skool
    Writing back_to_skool/save/save.html
    Writing disassembly files in back_to_skool/save
  Parsing ../src/bts-start.skool
    Writing back_to_skool/start/start.html
    Writing disassembly files in back_to_skool/start
  Writing back_to_skool/index.html
Using ref file: ../src/csc.ref
Parsing ../src/csc.skool
Creating directory html/contact_sam_cruise
Copying ../src/style.css to html/contact_sam_cruise/style.css
Copying ../src/skool.js to html/contact_sam_cruise/skool.js
  Wrote contact_sam_cruise/images/logo.png
  Writing disassembly files in contact_sam_cruise/asm
  Writing contact_sam_cruise/maps/all.html
  Writing contact_sam_cruise/maps/routines.html
  Writing contact_sam_cruise/maps/data.html
  Writing contact_sam_cruise/maps/messages.html
  Writing contact_sam_cruise/graphics/playarea.html
  Writing contact_sam_cruise/graphics/asstart.html
  Writing contact_sam_cruise/graphics/as.html
  Writing contact_sam_cruise/graphics/astiles/astiles.html
  Writing contact_sam_cruise/buffers/cbuffer.html
  Writing contact_sam_cruise/tables/keys.html
  Writing contact_sam_cruise/maps/command_lists.html
  Writing contact_sam_cruise/buffers/gbuffer.html
  Writing contact_sam_cruise/graphics/graphics.html
  Writing contact_sam_cruise/graphics/glitches.html
  Writing contact_sam_cruise/reference/bugs.html
  Writing contact_sam_cruise/reference/facts.html
  Writing contact_sam_cruise/reference/glossary.html
  Writing contact_sam_cruise/reference/pokes.html
  Parsing ../src/csc-load.skool
    Writing contact_sam_cruise/load/load.html
    Writing disassembly files in contact_sam_cruise/load
  Parsing ../src/csc-save.skool
    Writing contact_sam_cruise/save/save.html
    Writing disassembly files in contact_sam_cruise/save
  Parsing ../src/csc-start.skool
    Writing contact_sam_cruise/start/start.html
    Writing disassembly files in contact_sam_cruise/start
  Writing contact_sam_cruise/index.html"""

class DisassembliesTest(SkoolKitTestCase):
    def setUp(self):
        rmtree(ODIR, True)

    def tearDown(self):
        rmtree(ODIR, True)

    def test_html(self):
        # Check whether sd.ref, bts.ref and csc.ref are present
        disassemblies = (
            ('sd.ref', 'skool_daze'),
            ('bts.ref', 'back_to_skool'),
            ('csc.ref', 'contact_sam_cruise')
        )
        ref_files = []
        html_dirs = []
        for r, d in disassemblies:
            ref_file = '../src/%s' % r
            if os.path.isfile(ref_file):
                ref_files.append(ref_file)
                html_dirs.append(d)
        if len(ref_files) != len(disassemblies):
            raise nose.SkipTest()

        # Write the disassemblies
        cmdline, output, error = self.run_skool2html('-i null -d %s %s' % (ODIR, ' '.join(ref_files)))
        self.assert_no_errors(error, cmdline)
        self.assert_output_equal(output, OUTPUT.split('\n'))

        # Do XHTML validation using lxml.etree
        f = open(XHTML_XSD)
        xmlschema_doc = etree.parse(f)
        f.close()
        xmlschema = etree.XMLSchema(xmlschema_doc)
        for root, dirs, files in os.walk(ODIR):
            for fname in files:
                if fname[-5:] == '.html':
                    htmlfile = join(root, fname)
                    xhtml = etree.parse(htmlfile)
                    xmlschema.assertValid(xhtml)

        # Check links using linklint (if present)
        try:
            _, output, error = self.run_cmd('which linklint')
        except:
            output = None
        if output:
            linklint = output[0]
            doc_dir = join(ODIR, 'doc')
            dirs = ['/%s/@' % d for d in html_dirs]
            ll_cmdline, output, error = self.run_cmd('%s %s -limit 5000 -quiet -root %s -doc %s' % (linklint, ' '.join(dirs), ODIR, doc_dir), err_lines=True)
            no_errors = 'No errors, no warnings.'
            self.assertEqual(error[-1][:len(no_errors)], no_errors, 'linklint: %s' % error[-1])

if __name__ == '__main__':
    unittest.main()
